@@ -70,11 +70,9 @@ public:
     void    readSettings(QSettings * settings);
     void    saveSettings(QSettings * settings);
 
-    void    setLnbLo(double freq_mhz);
     double  lnbLo();
     void    readLnbLoFromSettings(QSettings * settings);
 
-    void    setGain(QString &name, double value);
     double  gain(QString &name);
 
     void    setAgc(bool enabled);
@@ -99,7 +97,12 @@ public:
     void    setAntenna(const QString &antenna);
 
     void    setGainStages(gain_list_t &gain_list);
-    void    restoreManualGains(QSettings *settings);
+    void    restoreManualGains(void);
+
+    void    setFreqCtrlReset(bool enabled);
+
+public slots:
+    void    setGain(QString name, double value);
 
 signals:
     void gainChanged(QString name, double value);
@@ -111,6 +114,10 @@ signals:
     void iqBalanceChanged(bool enabled);
     void ignoreLimitsChanged(bool ignore);
     void antennaSelected(QString antenna);
+    void freqCtrlResetChanged(bool enabled);
+
+public slots:
+    void setLnbLo(double freq_mhz);
 
 private slots:
     void on_lnbSpinBox_valueChanged(double value);
@@ -121,6 +128,7 @@ private slots:
     void on_iqBalanceButton_toggled(bool checked);
     void on_ignoreButton_toggled(bool checked);
     void on_antSelector_currentIndexChanged(const QString &antenna);
+    void on_freqCtrlResetButton_toggled(bool checked);
 
     void sliderValueChanged(int value);
 
