@@ -39,8 +39,8 @@ public:
     void setTooltipsEnabled(bool enabled) { m_TooltipsEnabled = enabled; }
     void setBookmarksEnabled(bool enabled) { m_BookmarksEnabled = enabled; }
 
-    void setNewFttData(float *fftData, int size);
-    void setNewFttData(float *fftData, float *wfData, int size);
+    void setNewFftData(float *fftData, int size);
+    void setNewFftData(float *fftData, float *wfData, int size);
 
     void setCenterFreq(quint64 f);
     void setFreqUnits(qint32 unit) { m_FreqUnits = unit; }
@@ -70,6 +70,12 @@ public:
         drawOverlay();
     }
 
+    void getHiLowCutFrequencies(int *LowCut, int *HiCut)
+    {
+        *LowCut = m_DemodLowCutFreq;
+        *HiCut = m_DemodHiCutFreq;
+    }
+
     void setDemodRanges(int FLowCmin, int FLowCmax, int FHiCmin, int FHiCmax, bool symetric);
 
     /* Shown bandwidth around SetCenterFreq() */
@@ -121,6 +127,7 @@ signals:
     void newHighCutFreq(int f);
     void newFilterFreq(int low, int high);  /* substitute for NewLow / NewHigh */
     void pandapterRangeChanged(float min, float max);
+    void newZoomLevel(float level);
 
 public slots:
     // zoom functions
@@ -134,6 +141,7 @@ public slots:
     void setFftFill(bool enabled);
     void setPeakHold(bool enabled);
     void setFftRange(float min, float max);
+    void setWfColormap(const QString &cmap);
     void setPandapterRange(float min, float max);
     void setWaterfallRange(float min, float max);
     void setPeakDetection(bool enabled, float c);
